/**
 * @file        ADSO-9513
 * @description Create account
 * @author      Pethaperumal Natarajan (Pethaperumal.NATARAJAN.intern@3ds.com)
 * @copyright   Dassault Systèmes
 */

import { qCustomMatcher } from '../../e2elib/lib/src/main/qmatchers.const';
import { AppMP, Demo, Scenario } from '../../libmp/appmp';
import { ListAccount, ListAccountColumn, ListAccountContextMenuItem, ListAccountTypeImage } from '../../libmp/forms/form.account';
import { DialogAccount, DialogAccountFields } from '../../libmp/dialogs/dialog.account';
import { StepList } from '../../libappsop/listsop';
import { StepDialog } from '../../libappsop/dialogsop';
import { DataFoodBaseAccountCostDriver, DataFoodBaseAccountName, DataFoodBaseAccountTimeUnit } from '../../libmp/data/data.account';

describe('ADSO-9513 - Create account', () => {
  const appMp = AppMP.getInstance();
  const listAccount = appMp.viewAccount.frmAccount.listAccount;
  let dlgAccount: DialogAccount;
  const newAccountValue: DialogAccountFields = {
    AccountType: DataFoodBaseAccountName.Preference,
    ReportType: 'Report 1',
    Budget: '1000',
    CostDriver: DataFoodBaseAccountCostDriver.Fixed,
    Cost: '999',
    TimeUnit: DataFoodBaseAccountTimeUnit.Month,
    LengthOfTime: '1',
  };
  const newRow: ListAccountColumn = {
    'Account type': ListAccountTypeImage.Preferences,
    'Report type': 'Report 1',
    Name: 'New account',
    'Default cost driver': DataFoodBaseAccountCostDriver.Fixed,
    'Default cost': '999',
    'Time unit': DataFoodBaseAccountTimeUnit.Month,
    Budget: '1,000',
  };

  beforeAll(async () => {
    jasmine.addMatchers(qCustomMatcher);
    await appMp.login();
  });

  afterAll(async () => {
    await appMp.resetActiveView(appMp.viewAccount);
    await appMp.cleanupAndLogout();
  });

  afterEach(async () => {
    await appMp.checkToastMessage();
  });

  it(`Step 1 - ${AppMP.getDemoDataPath(Demo.Food, Scenario.Base)}`, async () => {
    await appMp.createDemoDataset(Demo.Food, Scenario.Base, false);
  });

  it(`Step 2 - Open view ${appMp.viewAccount.viewPath}.`, async () => {
    await appMp.viewAccount.switchTo();
  });

  it(`Step 3 - ${StepList.selectRowAndClickMenu(ListAccount.title, { Name: DataFoodBaseAccountName.Margin }, ListAccountContextMenuItem.Create.Label)}`, async () => {
    const row = await listAccount.getRow({ Name: DataFoodBaseAccountName.Margin });
    [dlgAccount] = await listAccount.selectContextMenu(ListAccountContextMenuItem.Create, row);
  });

  it(`Step 4 - ${StepDialog.verifyDialogValues(DialogAccount.title, {
    ParentAccountName: DataFoodBaseAccountName.Margin,
    AccountType: DataFoodBaseAccountName.Margin,
  })}`, async () => {
    await dlgAccount.verifyDialogValues({ ParentAccountName: DataFoodBaseAccountName.Margin, AccountType: DataFoodBaseAccountName.Margin });
  });

  it(`Step 5 - ${StepDialog.verifyDialogEnabled(DialogAccount.title, { AccountType: 'false' })}`, async () => {
    await dlgAccount.verifyDialogEnabled({ AccountType: 'false' });
  });

  it(`Step 6 - ${StepDialog.verifyOKDisabled()}`, async () => {
    await dlgAccount.verifyOKDisabled();
  });

  it(`Step 7 - ${StepDialog.updateDialogValues(DialogAccount.title, { ParentAccountName: DataFoodBaseAccountName.Empty, Name: 'New account' })}`, async () => {
    await dlgAccount.updateDialogValues({ ParentAccountName: DataFoodBaseAccountName.Empty, Name: 'New account' });
  });

  it(`Step 8 - ${StepDialog.verifyDialogEnabled(DialogAccount.title, { AccountType: 'true' })}`, async () => {
    await dlgAccount.verifyDialogEnabled({ AccountType: 'true' });
  });

  it(`Step 9 - ${StepDialog.updateDialogValues(DialogAccount.title, newAccountValue)}`, async () => {
    await dlgAccount.updateDialogValues(newAccountValue);
  });

  it(`Step 10 - ${StepDialog.clickOK()}`, async () => {
    await dlgAccount.clickOK();
  });

  it(`Step 11 - ${StepList.verifyRowExists(ListAccount.title, newRow)}`, async () => {
    await listAccount.verifyRowExists(newRow);
  });
});
